import { createContext, useContext, useState } from "react";
import * as api from "../lib/api";
import { toast } from "react-toastify";
export const AppContext = createContext();
export const useAppContext = () => useContext(AppContext);

export function ContextProvider({ children }) {
  const [userSeats, setUserSeats] = useState([]);
  const [reservedSeats, setReservedSeats] = useState([]);
  const [seatLayout, setSeatLayout] = useState([]);
  const [isLoading, setisLoading] = useState(false);

  const loginOrRegister = async ({ mobile }) => {
    try {
      const user = await api.getUserByMobile({ mobile });
      if (!user.length) {
        //register
        const response = await api.register({ mobile });
        sessionStorage.setItem("userId", response?.id);
        await getAllReservedSeats();
        // return true for  newly registered user
        return true;
      } else {
        //login
        sessionStorage.setItem("userId", user?.[0]?.id);
        const response = await getAllReservedSeats();
        const userSelectedSeats = response.filter((e) => e.id == user[0]?.id);
        sessionStorage.setItem(
          "seatLimit",
          userSelectedSeats?.[0]?.seats?.length
        );
        // return false for  already registered user
        return false;
      }
    } catch (err) {
      toast.error(err.message);
    }
  };
  const addOrUpdateSeats = async ({ id, seats }) => {
    try {
      //check user already booked seats or not
      if (!userSeats.length) {
        //add seats into reserved seats
        const response = await api.addSeats({ seats });
        setReservedSeats([...reservedSeats, response]);
      } else {
        //update user seats only
        const response = await api.updateSeats({ id, seats });
        setUserSeats(response.seats);
      }
    } catch (err) {
      toast.error(err.message);
    }
  };
  const getAllReservedSeats = async () => {
    setisLoading(false);
    try {
      //to clear the data for glitch
      setReservedSeats([]);
      setUserSeats([]);
      const response = await api.getAllReservedSeats();
      const userid = sessionStorage.getItem("userId");

      response.map((e) =>
        e.id != userid
          ? // condition1=get all reserved seats except logged user
            setReservedSeats((prevReservedSeat) => [
              ...prevReservedSeat,
              ...e.seats,
            ])
          : // condition2=get only reserved seats of logged user
            setUserSeats(e.seats)
      );

      setisLoading(true);
      return await response;
    } catch (err) {
      toast.error(err.message);
    }
  };

  const getSeatLayout = async () => {
    setisLoading(false);
    try {
      setSeatLayout([]);
      const response = await api.getSeatLayout();
      setSeatLayout(response);
      setisLoading(true);
    } catch (err) {
      toast.error(err.message);
    }
  };

  const value = {
    userSeats,
    reservedSeats,
    isLoading,
    seatLayout,
    loginOrRegister,
    addOrUpdateSeats,
    getAllReservedSeats,
    getSeatLayout,
  };
  return <AppContext.Provider value={value}>{children}</AppContext.Provider>;
}
